"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseBacklogService = void 0;
const constants_1 = require("../../constants");
const samples_1 = require("../../samples");
class BaseBacklogService {
    constructor(config) {
        this.config = config;
    }
    // Static functions
    static createSampleBacklogItems(empty = false) {
        return empty ? samples_1.emptyBacklogItems : samples_1.defaultBacklogItems;
    }
    // Base functions
    createBacklogItems(items) {
        return this.createProviderBacklogItems(items);
    }
    createSprints(sprints) {
        return __awaiter(this, void 0, void 0, function* () {
            sprints = sprints || this.createSprintsFromConfig();
            return this.createProviderSprints(sprints);
        });
    }
    // Private functions
    createSprintsFromConfig() {
        const { sprints: sprintConfig } = this.config;
        if (!sprintConfig) {
            throw new Error("Section backlog.sprints of cse.json is required for this operation");
        }
        const { startDate, lengthOfSprintInDays, numberOfSprints } = sprintConfig;
        const sprints = [];
        const timezoneOffset = new Date().getTimezoneOffset() * constants_1.NumberConstants.millisecondsInAMinute;
        let currentStartDate = new Date(Date.parse(startDate) + timezoneOffset);
        for (let i = 0; i < numberOfSprints; i++) {
            const finishDate = new Date(currentStartDate.getTime() + constants_1.NumberConstants.millisecondsInADay * lengthOfSprintInDays);
            sprints.push({
                name: `Sprint ${i + 1}`,
                startDate: currentStartDate,
                finishDate,
            });
            currentStartDate = finishDate;
        }
        return sprints;
    }
}
exports.BaseBacklogService = BaseBacklogService;
