"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AzureDevOpsBacklogService = void 0;
const azure_devops_node_api_1 = require("azure-devops-node-api");
const CoreApi_1 = require("azure-devops-node-api/CoreApi");
const WorkItemTrackingInterfaces_1 = require("azure-devops-node-api/interfaces/WorkItemTrackingInterfaces");
const WorkApi_1 = require("azure-devops-node-api/WorkApi");
const WorkItemTrackingApi_1 = require("azure-devops-node-api/WorkItemTrackingApi");
const retry_1 = require("../../../utils/retry");
const baseBacklogService_1 = require("../baseBacklogService");
class AzureDevOpsBacklogService extends baseBacklogService_1.BaseBacklogService {
    constructor(config) {
        super(config);
        this.getSprint = (id) => __awaiter(this, void 0, void 0, function* () {
            const teamContext = yield this.getTeamContext();
            const teamIteration = yield retry_1.retryAsync(() => this.workApi.getTeamIteration(teamContext, id));
            if (!teamIteration) {
                throw new Error(`Could not retrieve sprint ${id}`);
            }
            const { name, attributes } = teamIteration;
            return {
                id,
                name,
                startDate: attributes === null || attributes === void 0 ? void 0 : attributes.startDate,
                finishDate: attributes === null || attributes === void 0 ? void 0 : attributes.finishDate,
            };
        });
        this.createProviderBacklogItems = (items) => __awaiter(this, void 0, void 0, function* () {
            // await this.workItemTracking.createWorkItem()
            return items;
        });
        this.createProviderSprints = (sprints) => __awaiter(this, void 0, void 0, function* () {
            const teamContext = yield this.getTeamContext();
            return Promise.all(sprints.map((sprint) => __awaiter(this, void 0, void 0, function* () {
                // Creates classification node and returns identifier required for sprint ID
                const identifier = yield this.createClassificationNode(sprint);
                const { name, startDate, finishDate } = sprint;
                // Create new iteration
                const result = yield this.workApi.postTeamIteration({
                    id: identifier,
                    name,
                    attributes: {
                        startDate,
                        finishDate,
                    },
                }, teamContext);
                // Assign generated ID from Azure DevOps to sprint
                sprint.id = result.id;
                return sprint;
            })));
        });
        this.deleteSprint = (id) => __awaiter(this, void 0, void 0, function* () {
            const teamContext = yield this.getTeamContext();
            yield this.workApi.deleteTeamIteration(teamContext, id);
        });
        const { baseUrl, personalAccessToken, projectName } = config.providerOptions;
        const authHandler = azure_devops_node_api_1.getPersonalAccessTokenHandler(personalAccessToken);
        this.projectName = projectName;
        this.coreApi = new CoreApi_1.CoreApi(baseUrl, [authHandler]);
        this.workItemTracking = new WorkItemTrackingApi_1.WorkItemTrackingApi(baseUrl, [authHandler]);
        this.workApi = new WorkApi_1.WorkApi(baseUrl, [authHandler]);
    }
    createClassificationNode(sprint) {
        return __awaiter(this, void 0, void 0, function* () {
            const { name, startDate, finishDate } = sprint;
            const { identifier } = yield this.workItemTracking.createOrUpdateClassificationNode({
                name,
                attributes: {
                    startDate: startDate === null || startDate === void 0 ? void 0 : startDate.toISOString(),
                    finishDate: finishDate === null || finishDate === void 0 ? void 0 : finishDate.toISOString(),
                }
            }, this.projectName, WorkItemTrackingInterfaces_1.TreeStructureGroup.Iterations);
            if (!identifier) {
                throw new Error(`Was not able to retrieve identifier for ${this.projectName}`);
            }
            return identifier;
        });
    }
    getTeamContext() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.teamContext) {
                return this.teamContext;
            }
            const { name, id, defaultTeam } = yield this.coreApi.getProject(this.projectName);
            this.teamContext = {
                project: name,
                projectId: id,
                team: defaultTeam === null || defaultTeam === void 0 ? void 0 : defaultTeam.name,
                teamId: defaultTeam === null || defaultTeam === void 0 ? void 0 : defaultTeam.id,
            };
            return this.teamContext;
        });
    }
}
exports.AzureDevOpsBacklogService = AzureDevOpsBacklogService;
